// As of 2014-04-25, HTML panels are debugged from within the host app.
// Thus, Eclipse's built-in breakpoint support does not apply to HTML panels.
// A debugger window is opened in the host app when the panel is launched,
// but the host app must be started using "Debug As", and must start with the panel closed.
// (Note, however, that the debugging screen does not appear on some computers for unknown reasons.)

// Because the debugger window opens when the panel starts,
// there is no opportunity to set breakpoints before the panel starts running.
// Setting the debugPanel variable to true will add a "Start" button to the panel.
// The panel then won't run any code until this button is pressed, allowing you to add breakpoints first.

var debugPanel = false;

var MessageID =
{
    kErrorOperator                        :  0,
    kDeprecatedWebFontLogin               :  1,
    kDeprecatedWebFontLogout              :  2,
    kCeprecatedWebFontIsLoggedIn          :  3,
    kGetPanelSocketPort                   :  4,
    kDeprecatedIsRunningStandalone        :  5,
    kDeprecatedGetActivatedFonts          :  6,
    kDeprecatedGetDeactivatedFonts        :  7,
    kDesignActivateFamilyForFont          :  8,
    kEnableGoogleFonts                    :  9,
    kAreGoogleFontsEnabled                : 10,
    kGetDimensions                        : 11,
    kSetDimensions                        : 12,
    kDeleteFile                           : 13,
    kDeprecatedGetEULAAcceptance          : 14,
    kDeprecatedAcceptEULA                 : 15,
    kIsWindows                            : 16,
    kSaveDropDownSelection                : 17,
    kGetDropDownSelection                 : 18,
    kCouldNotCreateTmpFile                : 19,
    kConnectionComplete                   : 20,
    kDeprecatedWebINKLoggedInStateChanged : 21,
    kGoogleWebFontsEnabled                : 22,
    kGoogleWebFontsDisabled               : 23,
    kFontActivated                        : 24,
    kFontDeactivated                      : 25,
    kDeprecatedWebINKReplicationProgress  : 26,
    kDeprecatedWebINKReplicationStopped   : 27,
    kGoogleReplicationProgress            : 28,
    kGoogleReplicationStopped             : 29,
    kEnabled                              : 30,
    kDeprecatedIndexingProgress           : 31,
    kGetFontPNG                           : 32,
    kRenameSmartList                      : 33,
    kDeleteSmartList                      : 34,
    kQueryForSmartListFonts               : 35,
    kGetSmartList                         : 36,
    kGetPanelSmartListTree                : 37,
    kGetAllSmartLists                     : 38,
    kCreateSmartList                      : 39,
    kGetSmartListFonts                    : 40,
    kFontList                             : 41,
    kSetSmartListQuery                    : 42,
    kSmartListsChanged                    : 43,
    kSetSmartListName                     : 44,
    kFontMissing                          : 45,
    kFontActivationError                  : 46,
    kTrackFontPanelClick                  : 47,
    kUTSLogIn                             : 48,
    kUTSIsLoggedIn                        : 49,
    kUTSLoginStateChanged                 : 50,
    kRefreshUTSList                       : 51,
    kBulkReplicationProgress              : 52,
    kBulkReplicationStopped               : 53,
    kIncrementalReplicationProgress       : 54,
    kIncrementalReplicationStopped        : 55,
    kDataChanged                          : 56,
    kDeprecatedCanAccessWebINKFonts       : 57,
    kCanAccessGoogleFonts                 : 58,
    kForgetConnection                     : 59,
    kFontNonCompliant                     : 60,
    kGoOfflineNotification                : 61,
    kGoOnlineNotification                 : 62,
    kDownloadFinishedNotification         : 63,
    kTypekitEnabledNotification           : 64,
    kTypekitDisabledNotification          : 65,
    kStartPanel                           : 66,
    kSendHostAppKeyPress                  : 67,
    kFontServerReplicationProgress        : 68,
    kFontServerReplicationStopped         : 69
};

var MessageKey =
{
    kAddressKey              : "address",
    kCanAccessGoogleFontsKey : "canAccessGoogleFonts",
    kDeprecatedCanAccessWebINKFontsKey : "canAccessWebINKFonts",
    kConflictIDKey           : "conflictID",
    kDisplayNameKey          : "displayName",
    kFilenameKey             : "filename",
    kFontIDKey               : "fontID",
    kFontIDsKey              : "fontIDs",
    kFontImageKey            : "image",
    kFontLocationDesktopKey  : "desktop",
    kFontLocationGoogleKey   : "google",
    kFontLocationSystemKey   : "system",
    kFontLocationTypekitKey  : "typekit",
    kDeprecatedFontLocationWebINKKey   : "webINK",
    kFontsKey                : "fonts",
    kHeightKey               : "height",
    kHighPriorityFontIDsKey  : "highPriorityFontIDs",
    kIsConflictErrorKey      : "isConflictError",
    kIsEnabledKey            : "isEnabled",
    kIsLoggedInKey           : "isLoggedIn",
    kIsOnlineKey             : "isOnline",
    kIsReplicatingKey        : "isReplicating",
    kIsStandaloneKey         : "isStandalone",
    kIDeprecatedsWebINKOnlyKey         : "isWebINKOnly",
    kIsWindowsKey            : "isWindows",
    kListDefinitionKey       : "listDefinition",
    kMessageIDKey            : "messageID",
    kNumFontsKey             : "numFonts",
    kNumStepsKey             : "numSteps",
    kPNGFilenameKey          : "pngFilename",
    k2xPNGFilenameKey        : "2xPNGFilename",
    kPasswordKey             : "password",
    kPortKey                 : "port",
    kPostscriptNameKey       : "postscriptName",
    kReplicationModeKey      : "replicationMode",
    kReturnCodeKey           : "returnCode",
    kSavesSmartListKey       : "savesSmartList",
    kSelectionKey            : "selection",
    kServerKey               : "server",
    kServersKey              : "servers",
    kSetIDsKey               : "setIDs",
    kSmartListIDKey          : "smartListID",
    kSmartListKey            : "smartList",
    kSmartListNameKey        : "smartListName",
    kSmartListsKey           : "smartLists",
    kSmartSetIDsKey          : "smartSetIDs",
    kStepIndexKey            : "stepIndex",
    kTextKey                 : "text",
    kUsernameKey             : "username",
    kVersionKey              : "version",
    kWidthKey                : "width",
    kWindowHeightKey         : "windowHeight",
    kWindowWidthKey          : "windowWidth",
    kWorkgroupIDsKey         : "workgroupIDs",
    kIsCtrlPressedKey        : "isCtrlPressed",
    kIsShiftPressedKey       : "isShiftPressed",
    kIsAltPressedKey         : "isAltPressed",
    kKeyCodeKey              : "keyCode"
};

var FontLocation =
{
    kFontLocationGoogle     : "google",
    kFontLocationDesktop    : "desktop",
    kFontLocationSystem     : "system",
    kFontLocationTypekit    : "typekit"
};

var FontListIcon =
{
    kFontListIconGoogle      : "img/GoogleSourceTreeIcon.png",
    kFontListIconDesktop     : "img/Library.png",
    kFontListIconSystem      : "img/SystemFonts.png",
    kFontListIconTypekit     : "img/TypekitLibrary.png",
    kFontListIconMissing     : "img/FontMissing.png",
    kFontListIconError       : "img/FontError.png",
    kFontListIconUnavailable : "img/FontUnavailable.png",
    kFontIconOutOfCompliance : "img/OutOfCompliance.png"
};

var ProgressTitle = 
{
    kSynchronizeGoogle : localizeString("SynchronizeGoogle"),
    kSynchronizeUTS    : localizeString("SynchronizeUTS")
};

var DropdownIcon =
{
    kMyFontsUTC             : "./img/UTCDropDownIcon.png",
    kMyFontsStandalone      : "./img/LibraryDropDownIcon.png",
    kSystemFonts            : "./img/SystemFontsDropDownIcon.png",
    kTypekit                : "./img/TypekitDropDownIcon.png",
    kGoogle                 : "./img/GoogleDropDownIcon.png",
    kSmartListUTC           : "./img/UTCDropDownIcon.png",
    kSmartListStandalone    : "./img/SmartListDropDownIcon.png"
};

var DropdownTitle =
{
    kMyFontsWorkgroup       : localizeString("MyFonts"),
    kSystemFontsWorkgroup   : localizeString("SystemFontsLibraryName"),
    kTypekitWorkgroup       : localizeString("Typekit"),
    kGoogleWorkgroup        : localizeString("Google")
};

// Heights include any margin *above* the real content, but not *below*
var kSearchBarContainerHeight = 28;
var kToolbarContainerHeight   = 24;
var kBottomSpace              = 7;      // spacer we want at the bottom of the panel

var CoreErrorCodes =
{
    kSuccess  : 0,
    kNotFound : 117440512,
};

var ReplicationMode = 
{
    kGoogleWebFonts : 4
};

var SmartListIDs =
{
    kDesktopFonts     : 0xffffffff,
    kSystemFonts      : 0xfffffffc,
    kTypekit          : 0xfffffffb,
    kGoogle           : 0xfffffffd,
    kDeprecatedWebINK : 0xfffffffe
};

var ActivationMode =
{
    kInactive : 0,
    kMissing  : 7
};

var SmartListType =
{
    kLibrary         : "library",
    kMasterGoogle    : "google",
    kTypekit         : "typekit",
    kSystemLibrary   : "system-library",
    kStaticSet       : "static-set",
    kSystemOtherSet  : "system-other-set",
    kSystemUserSet   : "system-user-set",
    kSystemLocalSet  : "system-local-set",
    kSystemNetworkSet: "system-network-set",
    kSystemSet       : "system-set",
    kSmartSet        : "smart-set"
};

var IconFile = 
{
    kSet                            : "img/Set.png",
    kDisabledSet                    : "img/SetDisabled.png",
    kSmartSet                       : "img/SmartSet.png",
    kDisabledSmartSet               : "img/SmartSetDisabled.png",
    kLibrary                        : "img/Library.png",
    kGoogle                         : "img/GoogleSourceTreeIcon.png",
    kDisabledGoogle                 : "img/GoogleSourceTreeIconDisabled.png",
    kSystemFont                     : "img/SystemFonts.png",
    kSystemFontSet                  : "img/SystemFontsSet.png",
    kTypekit                        : "img/TypekitLibrary.png",
    kDisabledTypekit                : "img/TypekitLibraryDisabled.png"
};

const fontRowHeight = 30;
const fontRowWidth  = 400;

const dropDownMenuMaxHeight = 170;

var Keyboard = 
{
    BACKSPACE: 8,
    TAB: 9,
    ENTER: 13,
    SHIFT: 16,
    CTRL: 17,
    ALT: 18,
    PAUSE: 19,
    CAPS_LOCK: 20,
    ESCAPE: 27,
    SPACE: 32,
    PAGE_UP: 33,
    PAGE_DOWN: 34,
    END: 35,
    HOME: 36,
    LEFT_ARROW: 37,
    UP_ARROW: 38,
    RIGHT_ARROW: 39,
    DOWN_ARROW: 40,
    INSERT: 45,
    DELETE: 46,
    KEY_0: 48,
    KEY_1: 49,
    KEY_2: 50,
    KEY_3: 51,
    KEY_4: 52,
    KEY_5: 53,
    KEY_6: 54,
    KEY_7: 55,
    KEY_8: 56,
    KEY_9: 57,
    KEY_A: 65,
    KEY_B: 66,
    KEY_C: 67,
    KEY_D: 68,
    KEY_E: 69,
    KEY_F: 70,
    KEY_G: 71,
    KEY_H: 72,
    KEY_I: 73,
    KEY_J: 74,
    KEY_K: 75,
    KEY_L: 76,
    KEY_M: 77,
    KEY_N: 78,
    KEY_O: 79,
    KEY_P: 80,
    KEY_Q: 81,
    KEY_R: 82,
    KEY_S: 83,
    KEY_T: 84,
    KEY_U: 85,
    KEY_V: 86,
    KEY_W: 87,
    KEY_X: 88,
    KEY_Y: 89,
    KEY_Z: 90,
    LEFT_META: 91,
    RIGHT_META: 92,
    SELECT: 93,
    NUMPAD_0: 96,
    NUMPAD_1: 97,
    NUMPAD_2: 98,
    NUMPAD_3: 99,
    NUMPAD_4: 100,
    NUMPAD_5: 101,
    NUMPAD_6: 102,
    NUMPAD_7: 103,
    NUMPAD_8: 104,
    NUMPAD_9: 105,
    MULTIPLY: 106,
    ADD: 107,
    SUBTRACT: 109,
    DECIMAL: 110,
    DIVIDE: 111,
    F1: 112,
    F2: 113,
    F3: 114,
    F4: 115,
    F5: 116,
    F6: 117,
    F7: 118,
    F8: 119,
    F9: 120,
    F10: 121,
    F11: 122,
    F12: 123,
    NUM_LOCK: 144,
    SCROLL_LOCK: 145,
    SEMICOLON: 186,
    EQUALS: 187,
    COMMA: 188,
    DASH: 189,
    PERIOD: 190,
    FORWARD_SLASH: 191,
    GRAVE_ACCENT: 192,
    OPEN_BRACKET: 219,
    BACK_SLASH: 220,
    CLOSE_BRACKET: 221,
    SINGLE_QUOTE: 222
};

var panel = new Panel();
