function MessageHandler()
{
    this.handlersByMessageID = { };
    
    this.handleMessage = function(messageStr)
    {
        var message = JSON.parse(messageStr.slice(1, -1));
        var handlers = this.handlersByMessageID[message[MessageKey.kMessageIDKey]];
        
        if (handlers != null)
        {
            for (var handlerNum = 0; handlerNum < handlers.length; ++handlerNum)
            {
                handlers[handlerNum](message);
            }
        }
    };
    
    this.addHandlerForMessageID = function(messageID, handler)
    {
        var handlers = this.handlersByMessageID[messageID];
        
        if (handlers == null)
        {
            handlers = [];
            this.handlersByMessageID[messageID] = handlers;
        }
            
        handlers.push(handler);
    };
}

panel.messageHandler = new MessageHandler();

new CSInterface().addEventListener("com.extensis.PluginToPanel", function(event) {
    panel.messageHandler.handleMessage(event.data);
});

function registerHandlers()
{
    panel.messageHandler.addHandlerForMessageID(MessageID.kConnectionComplete,
        function(message)
        {
            panel.handleFirstLaunchInfo(message[MessageKey.kIsStandaloneKey]);        
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontList,
        function(message)
        {
            panel.fontList.fontListHandler(message[MessageKey.kFilenameKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kGetFontPNG,
        function(message)
        {
            panel.fontList.fontPNGHandler(message[MessageKey.kFontsKey]);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontActivated,
        function(message)
        {
            fontActivatedHandler(message[MessageKey.kFontIDsKey]);
        });    
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontDeactivated,
        function(message)
        {
            fontDeactivatedHandler(message[MessageKey.kFontIDsKey]);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontMissing,
        function(message)
        {
            fontMissingHandler(message[MessageKey.kFontIDsKey]);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontNonCompliant,
        function(message)
        {
            fontNonCompliantHandler(message[MessageKey.kFontIDsKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kFontActivationError,
        function(message)
        {
            if (message[MessageKey.kIsConflictErrorKey])
            {
                var fontIDs = []; 
                fontIDs[0] = message[MessageKey.kFontIDKey];

                fontActivationErrorHandler(fontIDs);
            }
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kDesignActivateFamilyForFont,
        function(message)
        {
            returnedErrorCodeHandler("DesignActivateFamilyForFont", message[MessageKey.kReturnCodeKey]);
        });
    
    // bulk replication messages
    panel.messageHandler.addHandlerForMessageID(MessageID.kBulkReplicationProgress,
        function(message)
        {
            bulkReplicationProgressHandler(message[MessageKey.kTextKey],
                                           message[MessageKey.kStepIndexKey],
                                           message[MessageKey.kNumStepsKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kBulkReplicationStopped,
        function(message)
        {
            bulkReplicationStoppedHandler();
        });
    
    // incremental replication messages
    panel.messageHandler.addHandlerForMessageID(MessageID.kIncrementalReplicationProgress,
        function(message)
        {
            incrementalReplicationProgressHandler(message[MessageKey.kTextKey],
                                                  message[MessageKey.kStepIndexKey],
                                                  message[MessageKey.kNumStepsKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kIncrementalReplicationStopped,
        function(message)
        {
            incrementalReplicationStoppedHandler();
        });
    
    // font server replication messages
    panel.messageHandler.addHandlerForMessageID(MessageID.kFontServerReplicationProgress,
        function(message)
        {
           fontServerReplicationProgressHandler(message[MessageKey.kTextKey],
                                                message[MessageKey.kStepIndexKey],
                                                message[MessageKey.kNumStepsKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kFontServerReplicationStopped,
        function(message)
        {
            fontServerReplicationStoppedHandler();
        });

    // Google replication messages
    panel.messageHandler.addHandlerForMessageID(MessageID.kGoogleReplicationProgress,
        function(message)
        {
            googleReplicationProgressHandler(message[MessageKey.kTextKey],
                                             message[MessageKey.kStepIndexKey],
                                             message[MessageKey.kNumStepsKey]);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kGoogleReplicationStopped,
        function(message)
        {
            googleReplicationStoppedHandler();
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kGetAllSmartLists,
        function(message)
        {
            panel.dropDownMenu.dropDownMenuHandler(message[MessageKey.kSmartListsKey]);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kCreateSmartList, 
        function(message)
        {
            var smartListID = parseInt(message[MessageKey.kSmartListKey][MessageKey.kSmartListIDKey]);
            var smartListName = message[MessageKey.kSmartListKey][MessageKey.kSmartListNameKey];
            panel.dropDownMenu.smartListCreatedHandler(smartListID, smartListName);
            panel.messageSender.getSmartListTree(smartListID);
        });
        
    panel.messageHandler.addHandlerForMessageID(MessageID.kGetDropDownSelection,
        function(message)
        {
            panel.dropDownMenu.dropDownSelectionHandler(message[MessageKey.kSelectionKey]);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kAreGoogleFontsEnabled,
        function(message)
        {
            if(message[MessageKey.kReturnCodeKey] == CoreErrorCodes.kSuccess)
            {
                panel.dropDownMenu.areGoogleFontsEnabledHandler(message[MessageKey.kIsEnabledKey], 
                    message[MessageKey.kIsReplicatingKey] && message[MessageKey.kReplicationModeKey] == ReplicationMode.kGoogleWebFonts);
            }
            else 
            {
                panel.dropDownMenu.areGoogleFontsEnabledHandler(false, false);
            }
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kUTSIsLoggedIn,
        function(message)
        {
            handleUTSIsLoggedIn(message[MessageKey.kIsLoggedInKey],
                message[MessageKey.kIsOnlineKey]);
        });


    panel.messageHandler.addHandlerForMessageID(MessageID.kGoogleWebFontsEnabled,
        function(message)
        {
            panel.messageSender.areGoogleFontsEnabled();
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kGoogleWebFontsDisabled,
        function(message)
        {
            panel.messageSender.areGoogleFontsEnabled();
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kUTSLoginStateChanged,
        function(message)
        {
            panel.messageSender.getUTSIsLoggedIn();
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kTypekitEnabledNotification,
        function(message)
        {
            panel.dropDownMenu.typeKitEnabledHandler(true);
        });

    panel.messageHandler.addHandlerForMessageID(MessageID.kTypekitDisabledNotification,
        function(message)
        {
            panel.dropDownMenu.typeKitEnabledHandler(false);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kGetPanelSmartListTree,
        function(message)
        {
            var base64String = message[MessageKey.kListDefinitionKey];
            var xmlString = Base64.decode(base64String);
            
            xmlString = xmlString.replace("System Fonts (LOCALIZE ME)",         localizeString("SystemFontsLibraryName"));
            xmlString = xmlString.replace("Google Library (LOCALIZE ME)",       localizeString("GoogleLibraryName"));
            xmlString = xmlString.replace("Font Library (LOCALIZE ME)",         localizeString("FontLibraryName"));
            xmlString = xmlString.replace("Typekit Library (LOCALIZE ME)",      localizeString("Typekit"));
            xmlString = xmlString.replace("Local Workgroup (LOCALIZE ME)",      localizeString("LocalPersonalWorkgroup"));

            xmlString = xmlString.replace("Activated (LOCALIZE ME)",            localizeString("ActivatedFonts"));
            xmlString = xmlString.replace("Auto-Activated Fonts (LOCALIZE ME)", localizeString("AutoActivatedFonts"));
            xmlString = xmlString.replace("Duplicate Fonts (LOCALIZE ME)",      localizeString("DuplicateFonts"));
            xmlString = xmlString.replace("Favorite Fonts (LOCALIZE ME)",       localizeString("FavoriteFonts"));
            xmlString = xmlString.replace("Recently Added Fonts (LOCALIZE ME)", localizeString("RecentlyAddedFonts"));
            
            var treeData = panel.x2js.xml_str2json( xmlString );
            
            panel.showEditSmartList(treeData.nodes.children);
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kEnabled,
        function(message)
        {
            var enabled = message[MessageKey.kIsEnabledKey];
            
            if (enabled)
            {
                $("#disabled-screen").hide();
            }
            else
            {
                $("#disabled-screen").show();
            }
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kGoOnlineNotification,
        function(message)
        {
            panel.goOnline();
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kGoOfflineNotification,
        function(message)
        {
            panel.goOffline();
        });
    
    panel.messageHandler.addHandlerForMessageID(MessageID.kDataChanged,
        function(message)
        {
            panel.reloadData();
        });
}
