/*
 * Panel contains global variables
 */

function Panel()
{
    this.fontList = new FontList();

    this.startupTimer = null;
    this.reloadDataTimer = null;
    this.panelWarningTimer = null;

    //How long should we wait for app to respond before declaring that the app is in text edit mode
    this.panelGUIBlockingDelayCheck = 500; 

    this.isStandalone = false;

    this.selectedRow = null;

    this.fontListFile = null;

    this.lastFirstRow = 0;
    this.lastLastRow = 0;
    this.requestedFontIDs = [];

    this.dropDownMenu = new DropDownMenu();
    this.fontDigestTree = new FontDigestTree();

    this.x2js = new X2JS();

    this.isOnline = true;

    this.progress = new Progress();

    this.messageSender = null;

    this.messageHandler = null;

    this.fontApplyTimer = null;

    this.fontActivateTimer = null;

    this.isBlockedByGUI = false;

    this.cachedIsPhotoshop = null;
    this.cachedIsIllustrator = null;

    this.isPhotoshop = function()
    {
        if (this.cachedIsPhotoshop == null)
        {
            this.cachedIsPhotoshop = new CSInterface().hostEnvironment.appName == "PHXS";
        }

        return this.cachedIsPhotoshop;
    };
    
    this.isIllustrator = function()
    {
        if (this.cachedIsIllustrator == null)
        {
            this.cachedIsIllustrator = new CSInterface().hostEnvironment.appName == "ILST";
        }

        return this.cachedIsIllustrator;
    };
    
    this.isInDesign = function()
    {
        if (this.cachedIsInDesign == null)
        {
            this.cachedIsInDesign = new CSInterface().hostEnvironment.appName == "IDSN";
        }

        return this.cachedIsInDesign;
    };

    this.isWindows = function()
    {
        var isWindows = false;

        var osInformation = new CSInterface().getOSInformation();
        isWindows = (osInformation.indexOf('Windows') != -1) ? true : false;

        return isWindows;
    };

    this.setSelectedRow = function(row)
    {
        if (this.selectedRow != null)
        {
            this.selectedRow.imageDiv.classList.remove("selected");
        }

        if (row != null)
        {
            row.imageDiv.classList.add("selected");
            this.selectedRow = row;
        }
    };

    this.handleFirstLaunchInfo = function(runningStandalone)
    {
        this.isStandalone = runningStandalone;
        this.enableSmartListButtons();
        startPanel();
    };

    this.isRunningStandalone = function()
    {
        return this.isStandalone;
    };

    /**
     * Shows an alert box containing a message and one or two buttons.
     * 
     * Caller can provide an exit function that will be called when the
     * appropriate button is pressed. The alert will be hidden automatically,
     * even if exitFunction1 is null.
     * 
     * If buttonTitle2 is null, the second button will be hidden.
     * 
     * If hoverMode is true, then this alert will be in hover mode:
     *   - It will hide itself if the mouse leaves the object
     *   - Button 1 will be hidden
     */
    this.showAlert = function(message, explanation, buttonTitle1,
        exitFunction1, buttonTitle2, exitFunction2, hoverMode)
    {
        if (typeof hoverMode === 'undefined') { hoverMode = null; }
        var alertMessage = document.getElementById("alert-message");
        var alertExplanation = document.getElementById("alert-explanation");
        var alertButton1 = document.getElementById("alert-button1");
        var alertButton2 = document.getElementById("alert-button2");

        alertMessage.innerHTML = message;

        if (explanation != null)
        {
            alertExplanation.innerHTML = explanation;
        }
        else
        {
            alertExplanation.innerHTML = "";
        }

        var realButtonTitle1 = buttonTitle1;
        if (buttonTitle1 == null)
        {
            realButtonTitle1 = localizeString("OK");
        }

        alertButton1.innerHTML = realButtonTitle1;
        $("#alert-button1").show();
        
        var runExitFunction1 = function()
        {
            panel.hideAlert();
            
            if (exitFunction1 != null)
            {
                exitFunction1();
            }
        }
        
        alertButton1.onclick = runExitFunction1;

        if (buttonTitle2 == null)
        {
            $("#alert-button2").hide();
        }
        else
        {
            $("#alert-button2").show();
            alertButton2.innerHTML = buttonTitle2;
            
            var runExitFunction2 = function()
            {
                panel.hideAlert();
                
                if (exitFunction2 != null)
                {
                    exitFunction2();
                }
            };
            
            alertButton2.onclick = runExitFunction2;
            
        }
        
        var alertScreen = document.getElementById("alert-screen");
        alertScreen.className = "alert-screen";
        
        $(alertScreen).keydown(function(e)
        {
            if (e.which == Keyboard.ENTER)
            {
                runExitFunction1();
            }
            else if (w.which == Keyboard.ESCAPE && buttonTitle2 != null)
            {
                runExitFunction2();
            }
            
            return false;
        });
        
        $(alertScreen).focus();

        if (hoverMode != null)
        {
            $(alertScreen).mouseleave(function() {
                panel.hideAlert();
            });

            $("#alert-button1").hide();
        }
    };
    
    this.hideAlert = function()
    {
        var alertScreen = document.getElementById("alert-screen");
        alertScreen.className = "alert-screen-hidden";
        $("#font-list").focus();
    }
    

    this.showAbout = function()
    {
        var aboutScreen = document.getElementById("about-screen");
        aboutScreen.className = "about-screen";
    };

    this.hideAbout = function()
    {
        var aboutScreen = document.getElementById("about-screen");
        aboutScreen.className = "about-screen-hidden";
    };

    this.getMouseX = function()
    {
        return this.mouseX;
    };

    this.getMouseY = function()
    {
        return this.mouseY;
    };

    this.setMouseX = function(x)
    {
        this.mouseX = x;
    };

    this.setMouseY = function(y)
    {
        this.mouseY = y;
    };

    this.setTooltipObject = function(object)
    {
        this.tooltipObject = object;
    };

    this.getTooltipObject = function()
    {
        return this.tooltipObject;
    };

    this.reloadData = function()
    {
        if (this.reloadDataTimer != null)
        {
            window.clearInterval(this.reloadDataTimer);
        }
        
        this.reloadDataTimer = window.setInterval(function() { panel.handleDataReload(); }, 1000);
    }
    
    this.handleDataReload = function()
    {
        if (!this.isBulkReplicating() && 
            !this.isIncrementallyReplicating() &&
            !this.isFontServerReplicating())
        {
            window.clearInterval(this.reloadDataTimer);
            this.reloadDataTimer = null;

            if (this.dropDownMenu.currentSmartListIsReloadable())
            {
                this.fontList.reloadPreservingSelection();

                if (this.isSmartListPaneVisible())
                {
                    this.messageSender.getSmartListTree(this.dropDownMenu.savedSmartListID);
                }
            }
        }
    }
    
    this.showEditSmartList = function(treeData)
    {
        var smartListEdit = document.getElementById("smartlist-pane");
        smartListEdit.className = "smartlist-pane";

        var dropDownContainer = document.getElementById("dropdown-container");
        dropDownContainer.className = "dropdown-container-hidden";

        var itemName = this.dropDownMenu.getSelectedSmartListName();
        var nameInput = document.getElementById("smartlist-name-input");
        nameInput.value = itemName;
        this.focusSmartListNameInput();

        this.fontDigestTree.reloadTree(treeData);

        // Resize everything since the Done button has an incorrect width while
        // hidden
        resizeElements();
    };

    this.focusSmartListNameInput = function()
    {
        var nameInput = document.getElementById("smartlist-name-input");
        nameInput.focus();
        nameInput.select();
    };

    this.closeEditSmartList = function()
    {
        var smartListEdit = document.getElementById("smartlist-pane");
        smartListEdit.className = "smartlist-pane-hidden";

        var dropDownContainer = document.getElementById("dropdown-container");
        dropDownContainer.className = "dropdown-container";

        // Resize everything now that we've changed views
        resizeElements();
    };

    this.isSmartListPaneVisible = function()
    {
        var smartList = document.getElementById("smartlist-pane");

        return smartList.className == "smartlist-pane";
    };

    this.smartListDoneButtonPressed = function()
    {
        if (this.applyNewSmartListName())
        {
            this.closeEditSmartList();
        }
        else
        {
            this.showAlert("", localizeString("ErrorEmptyListName"), null,
                function()
                {
                    panel.focusSmartListNameInput();
                }, null, null);
        }
    };

    this.smartListKeyPress = function(e)
    {
        if (e.keyCode == 13)
        {
            this.smartListDoneButtonPressed();
            return false;
        }
        
        return true;
    };

    /*
     * Applies the smart list name to the database for the currently selected
     * item. Returns true if successful, or false if the name is of zero length
     */
    this.applyNewSmartListName = function()
    {
        var nameInput = document.getElementById("smartlist-name-input");
        var newItemName = nameInput.value.trim();

        if (newItemName.length == 0)
        {
            return false;
        }
        else
        {
            var oldItemName = this.dropDownMenu.getSelectedSmartListName();

            if (oldItemName != newItemName)
            {
                var smartListID = this.dropDownMenu.savedSmartListID;

                this.messageSender.setSmartListName(smartListID, newItemName);

                // Update the selected item text first, then re-request the
                // whole list
                this.dropDownMenu.setSmartListName(smartListID, newItemName);
                panel.messageSender.getAllSmartLists();
            }

            return true;
        }
    };

    this.updateFontCount = function()
    {
        var fontCountElement = document.getElementById("font-count");

        var countString = localizeString("FontCountLabel");
        countString = countString.replace('{0}', this.fontList.fontRows.length);

        fontCountElement.innerHTML = countString;
    };

    this.setUTSIsLoggedIn = function(loggedIn, isOnline)
    {
        var loggedOutScreen = document.getElementById("utc-nologin-screen");
        if (loggedIn)
        {
            $("#utc-nologin-screen").hide();
        }
        else
        {
            $("#utc-nologin-screen").show();
        }

        this.isOnline = isOnline;

        if (isOnline)
        {
            this.enableSmartListButtons();
        }
        else
        {
            this.disableSmartListButtons();
        }
    };

    this.isBulkReplicating = function()
    {
        return this.progress.bulkReplicationProgressInfo.progressRunning;
    }
    
    this.isIncrementallyReplicating = function()
    {
        return this.progress.incrementalReplicationProgressInfo.progressRunning;
    }
    
    this.isFontServerReplicating = function()
    {
        return this.progress.fontServerReplicationProgressInfo.progressRunning;
    }

    this.goOnline = function()
    {
        this.isOnline = true;

        this.enableSmartListButtons();

        if (this.dropDownMenu.currentSmartListIsReloadable())
        {
            this.fontList.reloadPreservingSelection();
        }
    }

    this.goOffline = function()
    {
        this.isOnline = false;

        this.closeEditSmartList();

        this.disableSmartListButtons();

        if (!this.isBulkReplicating()
            && this.dropDownMenu.currentSmartListIsReloadable())
        {
            this.fontList.reloadPreservingSelection();
        }
    }

    this.enableSmartListButtons = function()
    {
        var buttons = [ "edit-smart-list-button",
                        "create-smart-list-button",
                        "delete-smart-list-button" ];

        for (var buttonNum = 0; buttonNum < buttons.length; ++buttonNum)
        {
            this.enableSmartListButton(buttons[buttonNum]);
        }
    };

    this.enableSmartListButton = function(identifier)
    {
        var button = document.getElementById(identifier);

        var tooltips = { "edit-smart-list-button"   : localizeString("EditFontDigest"),
                         "create-smart-list-button" : localizeString("NewFontDigest"),
                         "delete-smart-list-button" : localizeString("DeleteFontDigest") };
        
        $(button).prop("disabled", false);

        if (button.tooltip == null)
        {
            var tooltip = document.createElement("span");
            tooltip.setAttribute("class", "tooltip");
            button.tooltip = tooltip;
        }

        button.tooltip.setAttribute("tooltip", tooltips[identifier]);
        
        $(button).prev(".button-mouse-area").hide();
    }
    
    this.disableSmartListButtons = function()
    {
        var buttons = [ "edit-smart-list-button",
                        "create-smart-list-button",
                        "delete-smart-list-button" ];

        for (var buttonNum = 0; buttonNum < buttons.length; ++buttonNum)
        {
            this.disableSmartListButton(buttons[buttonNum]);
        }
    };
    
    this.disableSmartListButton = function(identifier)
    {
        var button = document.getElementById(identifier);
        
        var tooltips = { "edit-smart-list-button"   : localizeString("EditFontDigest"),
                         "create-smart-list-button" : localizeString("NewFontDigest"),
                         "delete-smart-list-button" : localizeString("DeleteFontDigest") };
        
        if (!this.isOnline)
        {
            var tooltips = { "edit-smart-list-button"   : localizeString("EditFontDigestOffline"),
                             "create-smart-list-button" : localizeString("NewFontDigestOffline"),
                             "delete-smart-list-button" : localizeString("DeleteFontDigestOffline") };
        }

        $(button).prop("disabled", true);

        if (button.tooltip == null)
        {
            var tooltip = document.createElement("span");
            tooltip.setAttribute("class", "tooltip");
            button.tooltip = tooltip;
        }

        button.tooltip.setAttribute("tooltip", tooltips[identifier]);
        
        $(button).prev(".button-mouse-area").show();
    }
}
