function FontDigestTree()
{
    this.treeInitialized = false;
    this.loadTree = function(treeData)
    {
        $("#font-digest-tree").fancytree(
        {
            checkbox: true,
            selectMode: 2,
            source: treeData,
            autoscroll: true,
            click: function(event, data)
            {
                // We should not toggle, if target was "checkbox", because this
                // would result in double-toggle (i.e. no toggle)
                if( data.targetType === "title" )
                {
                    data.node.toggleSelected();
                }
            },
            select: function(event, data)
            {
                var workgroupIDs = [];
                var setIDs       = [];
                var smartSetIDs  = [];
                
                var smartListID = panel.dropDownMenu.savedSmartListID;  
                
                data.tree.visit(function(node)
                {
                    var id = parseInt(node.data.id);
                    
                    if (node.selected)
                    {
                        switch(node.data.type)
                        {
                            case SmartListType.kLibrary:
                            case SmartListType.kMasterGoogle:
                            case SmartListType.kSystemLibrary:
                            case SmartListType.kTypekit:
                                workgroupIDs.push(id);
                                break;
                                
                            case SmartListType.kStaticSet:
                            case SmartListType.kSystemOtherSet:
                            case SmartListType.kSystemUserSet:
                            case SmartListType.kSystemLocalSet:
                            case SmartListType.kSystemNetworkSet:
                            case SmartListType.kSystemSet:
                                setIDs.push(id);
                                break;
                                
                            case SmartListType.kSmartSet:
                                smartSetIDs.push(id);
                                break;
                        }
                    }
                });
                
                panel.messageSender.setSmartListQuery(
                    smartListID,
                    workgroupIDs,
                    setIDs,
                    smartSetIDs
                );
            },
        });
        
        this.treeInitialized = true;
    };
    
    this.reloadTree = function(treeData)
    {
        this.convertTreeData(treeData);
        if(this.treeInitialized)
        {
            var tree = $("#font-digest-tree").fancytree("getTree");
            tree.reload(treeData);
        }
        else
        {
            this.loadTree(treeData);
        }
    };
    
    this.convertTreeData = function(treeData)
    {
        for(var i = 0; i < treeData.length; ++i)
        {
            treeData[i].selected = treeData[i].selected == "1" ? true : false;
            treeData[i].icon = this.getIconForType(treeData[i].type, treeData[i].enabled);
            treeData[i].unselectable = treeData[i].enabled == "true" ? false : true;
            
            if(treeData[i].enabled == "false")
            {
                treeData.splice(i, 1);
                // Next node is now at this index, so we need to go back one to check it. Duh.
                --i;
            }
            else if(treeData[i].children)
            {
                if(!$.isArray(treeData[i].children))
                {
                    var array = [treeData[i].children];
                    treeData[i].children = array;
                }
                this.convertTreeData(treeData[i].children);
            }
        }
    };
    
    this.getIconForType = function(type, enabled)
    {
        var icon;
        
        switch(type)
        {
            case SmartListType.kStaticSet:
                if(enabled == "true")
                {
                    icon = IconFile.kSet;
                }
                else
                {
                    icon = IconFile.kDisabledSet;
                }
                break;
            
            case SmartListType.kSystemOtherSet:
            case SmartListType.kSystemUserSet:
            case SmartListType.kSystemLocalSet:
            case SmartListType.kSystemNetworkSet:
            case SmartListType.kSystemSet:
                icon = IconFile.kSystemFontSet;
                break;
            
            case SmartListType.kSmartSet:
                if(enabled == "true")
                {
                    icon = IconFile.kSmartSet;
                }
                else
                {
                    icon = IconFile.kDisabledSmartSet;
                }
                break;
            
            case SmartListType.kLibrary:
                icon = IconFile.kLibrary;
                break;
            
            case SmartListType.kMasterGoogle:
                if(enabled == "true")
                {
                    icon = IconFile.kGoogle;
                }
                else
                {
                    icon = IconFile.kDisabledGoogle;
                }
                break;
            
            case SmartListType.kTypekit:
                if(enabled == "true")
                {
                    icon = IconFile.kTypekit;
                }
                else
                {
                    icon = IconFile.kDisabledTypekit;
                }
                break;
            
            case SmartListType.kSystemLibrary:
                icon = IconFile.kSystemFont;
                break;               
        }
        
        return icon;
    };
};
