

function FontRow()
{
    this.imageTD      = document.createElement("td");
    this.missing      = false;
    this.nonCompliant = false;
    this.activated    = false;
    this.unavailable  = false;
    this.hasError     = false;
    
    this.setFontID = function(fontID)
    {
        this.fontID = fontID;
        
        this.imageDiv = document.createElement("div");
        this.imageDiv.setAttribute("id", this.fontID);
        this.imageDiv.classList.add("spacer");
        
        this.imageDiv.onclick = function()
        {
            panel.fontList.clearApplyTimer();
            
            if (panel.isBlockedByGUI)
            {
                panel.showAlert("",
                    localizeString("PanelDisabledDueToInlineEdit"),
                    localizeString("OK"),
                    null,
                    null,
                    null
                );
                
                return;
            }
            
            var row = panel.fontList.rowsByFontID[this.id];
            
            panel.setSelectedRow(row);
            
            if(row != null)
            {
                var displayName = row.getDisplayName();
                var postscriptName = row.getPostscriptName();

                if (!row.isLocalOrOnline())
                {
                    panel.showAlert("",
                        localizeString("FontUnavailableOffline"),
                        localizeString("OK"),
                        null,
                        null,
                        null
                    );                    
                }
                else if (row.isActive())
                {
                    //  ready to apply
                    var fontObject = {fontName: postscriptName, singleLayerLockedErrorString: localizeString("SingleLayerLockedError")};
                    new CSInterface().evalScript('applyFont('+ JSON.stringify(fontObject) +')', applyFontCallback);
                }
                else if (panel.fontList.canActivateFont(fontID, displayName, postscriptName))
                {
                    // Only request activation if we've already gotten a png
                    if(!activateFontIfReady(fontID))
                    {
                        // Start a timer to activate when we've gotten a png
                        clearInterval(panel.fontActivateTimer);
                        panel.fontActivateTimer = setInterval("activateFontIfReady(" + JSON.stringify(fontID) +")", 250);
                    }
                }
                else
                {
                    panel.showAlert("",
                        localizeString("ErrorActivatedFontWithSameName"),
                        localizeString("OK"),
                        null,
                        null,
                        null);
                    
                    //  Using setHasError here because that's what the flash panel does, but
                    //  setUnavailable makes more sense to me.  
                    row.setHasError(true);                    
                }
            }
        };

        this.imageDiv.onmouseover = function()
        {
            // This waits up to panel.panelGUIBlockingDelayCheck to set panel.isBlockedByGUI
            checkGUIBlockingPanel(); 

            // Set a timeout to run in panel.panelGUIBlockingDelayCheck + 100ms
            setTimeout(function() 
                {
                    if (panel.isBlockedByGUI)
                    {
                        panel.showAlert("",
                            localizeString("PanelDisabledDueToInlineEdit"),
                            null,
                            null,
                            null,
                            null,
                            true
                        );
                        
                        return;
                    }
                },
                (panel.panelGUIBlockingDelayCheck + 100)
            );
    
        }


        this.imageDiv.onmousemove = checkGUIBlockingPanel;
        
        // Tooltip 
        this.tooltip = document.createElement("span");
        this.tooltip.setAttribute("class", "tooltip");
        
        // Icon image
        this.fontLocationIconTD = document.createElement("td");
        this.fontLocationIconTD.classList.add("icon");
        this.fontLocationIcon = document.createElement("img");
        this.fontLocationIcon.classList.add("icon");
        
        this.placeholderText = document.createElement("p");
        this.placeholderText.classList.add("placeholder-text");
        this.placeholderText.appendChild(document.createTextNode(""));
        
        this.imageTD.appendChild(this.placeholderText);
        this.imageDiv.appendChild(this.fontLocationIconTD);
        this.imageDiv.appendChild(this.imageTD);
        
        this.installTooltips();
    };
    
    this.installTooltips = function()
    {
        $(this.imageDiv)
        .mouseenter(function() 
        {
            var row = panel.fontList.rowsByFontID[this.id];
            panel.setTooltipObject(row.imageTD);

            var fontList = document.getElementById("font-list");
            
            row.imageTD.appendChild(row.tooltip);
            row.imageTD.tooltip = row.tooltip;
            
            this.hoverTimer = setTimeout(function() 
            {
                $(row.tooltip).css("opacity", 1);
                
                var y = panel.getMouseY() + 20;
                var x = panel.getMouseX() + 15;
                
                if(y + row.tooltip.clientHeight > fontList.clientHeight)
                {
                    y = y - 40;
                }
                
                if(x + row.tooltip.clientWidth > fontList.clientWidth)
                {
                    x = fontList.clientWidth - row.tooltip.clientWidth;
                }
                
                $(row.tooltip).css("top", y);
                $(row.tooltip).css("left", x);
            }, 1000);
        })
        .mouseleave(function() 
        {
            var row = panel.fontList.rowsByFontID[this.id];
            
            clearTimeout(this.hoverTimer);

            if(row)
            {
                $(row.tooltip).css("opacity", 0);
                
                if ( row.tooltip && row.tooltip.parentNode === row.imageTD)
                {  
                    row.imageTD.removeChild(row.tooltip);
                } 
                panel.setTooltipObject(null);
            }
        });
    }

    this.simulateClick = function()
    {
        $(this.imageDiv).trigger("click");
    };
    
    this.getFontID = function()
    {
        return this.fontID;
    };

    this.setDisplayName = function(displayName)
    {
        this.displayName = displayName;
        this.tooltip.setAttribute("tooltip", this.getDisplayName());
        this.placeholderText.childNodes[0].nodeValue = displayName;
    };

    this.getDisplayName = function()
    {
        return this.displayName;
    };

    this.setPostscriptName = function(postscriptName)
    {
        this.postscriptName = postscriptName;
    };

    this.getPostscriptName = function()
    {
        return this.postscriptName;
    };

    this.setActive = function(activated)
    {
        if (this.missing)
        {
            this.reloadImage();
        }
        
        this.activated    = activated;
        this.missing      = false;
        this.unavailable  = false;

        if (activated)
        {
            panel.fontList.setActive(this.getFontID(), this.getDisplayName(), this.getPostscriptName());
        }
        else
        {
            panel.fontList.setInactive(this.getFontID(), this.getDisplayName(), this.getPostscriptName());
        }
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };

    this.isActive = function()
    {
        return this.activated;
    };
    
    this.updateFontListWithNewActivationStatus = function(activated)
    {
        if (activated)
        {
            panel.fontList.setActive(this.getFontID(), this.getDisplayName(), this.getPostscriptName());
        }
        else
        {
            panel.fontList.setInactive(this.getFontID(), this.getDisplayName(), this.getPostscriptName());
        }
    };
    
    this.setMissing = function(missing)
    {
        this.missing = missing;
        if (missing)
        {
            this.activated = false;
            this.updateFontListWithNewActivationStatus(this.activated);
        }
        
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };
    
    this.isMissing = function()
    {
        return this.missing;
    };

    this.setNonCompliant = function(isNonCompliant)
    {
        this.nonCompliant = isNonCompliant;
        
        if(this.nonCompliant)
        {
            this.activated = false;
            this.updateFontListWithNewActivationStatus(this.activated);
            
            panel.showAlert(localizeString("NonCompliantFontError"),
                localizeString("NonCompliantFontExplanation"),
                localizeString("OK"),
                null,
                null,
                null);
        }
                
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };
    
    this.isNonCompliant = function()
    {
        return this.nonCompliant;
    };
    
    this.setUnavailable = function(isUnavailable)
    {
        this.unavailable  = isUnavailable;

        if (this.unavailable)
        {
            panel.showAlert(localizeString("UnavailableFontError"),
                localizeString("UnavailableFontExplanation"),
                localizeString("OK"),
                null,
                null,
                null);
        }
        
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };
    
    this.isUnavailable = function()
    {
        return this.unavailable;
    };
    
    this.setHasError = function(hasError)
    {
        this.hasError     = hasError;
                        
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };
    
    this.isErroneous = function()
    {
        return this.hasError;
    };
        
    this.setFontLocation = function(fontLocation)
    {
        this.fontLocation = fontLocation;
        switch(fontLocation)
        {
            case FontLocation.kFontLocationGoogle:
                this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconGoogle);
                break;

            case FontLocation.kFontLocationDesktop:
                this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconDesktop);
                break;

            case FontLocation.kFontLocationSystem:
                this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconSystem);
                break;
            
            case FontLocation.kFontLocationTypekit:
                this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconTypekit);
                break;
        }

        this.fontLocationIconTD.appendChild(this.fontLocationIcon);
    };

    this.getFontLocation = function()
    {
        return this.fontLocation;
    };

    this.setIsLocal = function(isLocal)
    {
        this.isLocal = isLocal;
        
        this.determineImageVisibility();
        this.updateIcon();
        this.updateTooltip();
    };

    this.getIsLocal = function()
    {
        return this.isLocal;
    };
    
    this.isLocalOrOnline = function()
    {
        return this.getIsLocal() || panel.isOnline;
    };

    this.setImageURL = function(imageURL)
    {
        this.imageURL = imageURL;
        this.loadImage();
    };

    this.getImageURL = function()
    {
        return this.imageURL;
    };

    this.setImage = function(image)
    {
        this.image = image;
        this.loadImage();
    };

    this.getImage = function()
    {
        return this.image;
    };
    
    this.loadImage = function()
    {
        if((this.imageURL != null) || (this.image != null))
        {
            this.imageTD.setAttribute("id", this.fontID);
        }
        
        if((this.imageURL != null) && (this.image == null))
        {
            this.image = document.createElement("img");
            this.image.setAttribute("src", this.imageURL);
            this.image.setAttribute("height", fontRowHeight);
            this.image.setAttribute("width", fontRowWidth);
            this.image.setAttribute("class", "font-preview");
            
            panel.fontList.cachedFontImages[this.fontID] = this.image;
        }
        
        if(!this.isMissing() && this.isLocalOrOnline() && (this.image != null))
        {
            if ( this.placeholderText && this.placeholderText.parentNode === this.imageTD )
            {
                this.imageTD.removeChild(this.placeholderText);
            }

            this.imageTD.appendChild(this.image);
        }
    };
    
    this.determineImageVisibility = function()
    {
        if (this.isMissing() || this.isNonCompliant() || !this.isLocalOrOnline())
        {
            // If we have a text element then we need to remove the spacer image, otherwise
            // the font has gone missing after we've gotten pngs so we need to remove the
            // preview image
            var child = this.imageTD.childNodes[0];
            if (child.nodeName == "P")
            {
                this.imageTD.removeChild(this.placeholderText);
            }
            else
            {
                this.imageTD.removeChild(this.image);
            }


            // Switch the icon to the missing icon
            if (this.fontLocationIcon.parentNode != this.fontLocationIconTD)
            {
                this.fontLocationIconTD.appendChild(this.fontLocationIcon);
            }

            // Font name in red text
            this.placeholderText.style.color = "red";
            this.imageTD.appendChild(this.placeholderText);
        }
        else
        {
            try
            {
                this.fontLocationIconTD.removeChild(this.fontLocationIcon);
            }
            catch(e)
            {
            }

            this.tooltip.setAttribute("tooltip", this.getDisplayName());
            
            this.placeholderText.style.color = "#000000";
            this.setFontLocation(this.getFontLocation());
            this.loadImage();
        }
    };
    
    this.updateIcon = function()
    {
        if (this.fontLocationIcon.parentNode != this.fontLocationIconTD)
        {
            this.fontLocationTD.appendChild(this.fontLocationIcon);
        }
        
        if (this.isMissing())
        {
            this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconMissing);
        }
        else if (this.isNonCompliant())
        {
            this.fontLocationIcon.setAttribute("src", FontListIcon.kFontIconOutOfCompliance);
        }
        else if (this.isUnavailable())
        {
            this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconUnavailable);
        }
        else if (this.isErroneous())
        {
            this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconError);
        }
        else if (!this.isLocalOrOnline())
        {
            this.fontLocationIcon.setAttribute("src", FontListIcon.kFontListIconUnavailable);
        }
        else
        {
            this.setFontLocation(this.getFontLocation());
        }
    };
    
    this.updateTooltip = function()
    {
        if (this.isMissing())
        {
            var missingFontTooltip = localizeString("MissingFontTooltip");
            missingFontTooltip = missingFontTooltip.replace("{0}", this.getDisplayName());
            this.tooltip.setAttribute("tooltip", missingFontTooltip);
        }
        else if (this.isNonCompliant())
        {
            this.tooltip.setAttribute("tooltip", localizeString("NonCompliantFontError"));
        }
        else if (!this.isLocalOrOnline())
        {
            this.tooltip.setAttribute("tooltip", localizeString("FontUnavailableOffline"));
        }
        else
        {
            this.tooltip.setAttribute("tooltip", this.getDisplayName());
        }
    };
    
    this.reloadImage = function()
    {
        panel.requestedFontIDs[this.fontID] = this.fontID;
        panel.messageSender.getFontPNGs([ this.fontID ], [ this.fontID ], 0xffffffff);
    };
    
    
    this.matchesSearch = function(searchText)
    {
        return this.displayName.toLocaleLowerCase().indexOf(searchText) >= 0;
    };
    
    this.activateFont = function()
    {
        //  needs to be activated
        panel.messageSender.activateFont(this.fontID, this.postscriptName, this.displayName);

        clearInterval(panel.fontApplyTimer);
        panel.fontApplyTimer = setInterval("applyFontByID(" + JSON.stringify(this.fontID) +")", 500);
    };
}

function applyFontCallback(result)
{
    var error = result;
    
    if(panel.isInDesign())
    {
        var resultObject = JSON.parse(result);
        error = resultObject.succeeded;
    }
    
    if(!error)
    {
        console.log("Apply Font Error");
    }

}

function isFontActivatedCallback(result)
{
    var font = JSON.parse(result);
    if (font.isActivated)
    {
        //  ready to apply
        var row = panel.fontList.rowsByFontID[font.fontID];
        if(row != null)
        {
            var fontName = row.getPostscriptName();
            var fontObject = {fontName: fontName, singleLayerLockedErrorString: localizeString("SingleLayerLockedError")};
            new CSInterface().evalScript('applyFont('+ JSON.stringify(fontObject) +')', applyFontCallback);
            
            clearInterval(panel.fontApplyTimer);
        }
    }
}

function fontActivatedHandler(fontIDs)
{
    for (var i = 0; i < fontIDs.length; i++)
    {
        var fontID = fontIDs[i];
        var row = panel.fontList.rowsByFontID[fontID];
        if (row != null)
        {
            row.setActive(true);
        }
    }
}

function fontDeactivatedHandler(fontIDs)
{
    for (var i = 0; i < fontIDs.length; i++)
    {
        var row = panel.fontList.rowsByFontID[fontIDs[i]];
        if (row != null)
        {
            row.setActive(false);
        }
    }
}

function fontMissingHandler(fontIDs)
{
    for (var i = 0; i < fontIDs.length; i++)
    {
        var row = panel.fontList.rowsByFontID[fontIDs[i]];
        
        if (row != null)
        {
            row.setMissing(true);
        }
    }
}

function returnedErrorCodeHandler(functionName, errorCode)
{
    if (errorCode == CoreErrorCodes.kNotFound)
    {
        clearInterval(panel.fontApplyTimer);
        
        panel.showAlert("",
            "ErrorFontNotFound",
            localizeString("OK"),
            null,
            null,
            null);
    }
}

function fontActivationErrorHandler(fontIDs)
{
    clearInterval(panel.fontApplyTimer);
    
    for (var i = 0; i < fontIDs.length; i++)
    {
        var row = panel.fontList.rowsByFontID[fontIDs[i]];
        
        if (row != null)
        {
            panel.showAlert("",
                localizeString("ErrorFontConflict"),
                localizeString("OK"),
                null,
                null,
                null);
            
            row.setHasError(true);                    
        }
    }
}

function fontNonCompliantHandler(fontIDs)
{
    for (var i = 0; i < fontIDs.length; i++)
    {
        var row = panel.fontList.rowsByFontID[fontIDs[i]];
        
        if (row != null)
        {
            row.setNonCompliant(true);
        }
    }
}

function applyFontByID(fontID)
{
    var row = panel.fontList.rowsByFontID[fontID];
    if(row != null)
    {
        if (row.isActive())
        {
            var fontName = row.getPostscriptName();
            
            var fontObject = {name: fontName, id: fontID};
            new CSInterface().evalScript('isFontActivated('+ JSON.stringify(fontObject) +')', isFontActivatedCallback);
        }
    }
}

// Wait to have a png to activate the font
function activateFontIfReady(fontID)
{
    var activated = false;
    var row = panel.fontList.rowsByFontID[fontID];
    if(row != null)
    {
        if (row.image != null && row.image.getAttribute("src") != null)
        {
            clearInterval(panel.fontActivateTimer);
            row.activateFont();
            activated = true;
        }
    }
    return activated;
}

function checkGUIBlockingPanel()
{
    if (panel.isPhotoshop() && panel.panelWarningTimer == null)
    {
        // Set a fallback function to run (sets panel.isBlockedByGUI = true)
        panel.panelWarningTimer = setTimeout(
            function() 
            { 
                panel.isBlockedByGUI = true;
                panel.panelWarningTimer = null;
            },
            panel.panelGUIBlockingDelayCheck
        );

        // Ask CSInterface to run isGUIBlockingPanel() code, and pass it a callback
        new CSInterface().evalScript("isGUIBlockingPanel()",
            function(blocking)
            {
                clearTimeout(panel.panelWarningTimer);
                panel.panelWarningTimer = null;
                panel.isBlockedByGUI = blocking == "true";

            }
        );

    }
}
