var HOVER_FADE_TIME = 400;

var pasteboardBackgroundColor = jQuery.Color("#FFFFFF");
var pasteboardTextColor       = jQuery.Color("#000000");

var showInfo           = true;
var selectedIdentifier = null;

var bubbleCount       = 0;
var borderCount       = 0;
var printPreviewCount = 0;

var printFontLabel    = "";
var printTypeLabel    = "";
var printFoundryLabel = "";
var printClassLabel   = "";
var printFamilyLabel  = "";
var printVersionLabel = "";

var printRowNum       = 1;

var isMac             = false;

function setBackgroundColor (color)
{
    pasteboardBackgroundColor = jQuery.Color(color);
    $("body").css("background-color", color);
}

function setTextColor (color)
{
    pasteboardTextColor = jQuery.Color(color);
    $("body").css("color", color);
    rebuildUIElements();
}

function setIsMac()
{
    isMac = true;
}

function createSelectionUIElements(element, identifier, autoScroll, hover)
{
    var BUBBLE_HEIGHT     = 20;
    var BUBBLE_MAX_WIDTH  = 300;
    var BUBBLE_MARGIN     = 8;
    var BUBBLE_PADDING    = 20;

    var vScrollValue = $("body").scrollTop();
    var hScrollValue = $("body").scrollLeft();

    if (navigator.appName == "Microsoft Internet Explorer")
    {
        vScrollValue = $("html").scrollTop();
        hScrollValue = $("html").scrollLeft();
    }

    var divs         = [];
    var screenHeight = $(window).height();
    var screenWidth  = $(window).width();
    var position     = element.offset();
    var width        = element.outerWidth();
    var height       = element.outerHeight();

    var visibleTop    = position.top;
    var visibleBottom = position.top + height;
    var visibleLeft   = position.left;
    var visibleRight  = position.left + width;

    var totalBubbleHeight = BUBBLE_HEIGHT * 2 + BUBBLE_MARGIN;

    var parent = element.parent();

    while (parent.outerHeight() != null && parent.outerWidth() != null)
    {
        // For some reason, the <body> element has a height of 0 on Mac
        // On windows the body element is fine, but the html element must be skipped because it only has the window height
        if (parent.prop("tagName") != "BODY" &&
            parent.prop("tagName") != "HTML")
        {
            var parentOffset = parent.offset();

            if (parentOffset != null)
            {
                visibleBottom = Math.min(
                    visibleBottom,
                    parentOffset.top + parent.outerHeight()
                );

                visibleRight = Math.min(
                    visibleRight,
                    parentOffset.left + parent.outerWidth()
                );
            }
        }
        parent = parent.parent();
    }

    height = visibleBottom - visibleTop;
    width  = visibleRight  - visibleLeft;

    if (!hover)
    {
        var border = createSelectionBorder(
            position.left,
            position.top,
            width,
            height,
            element
        );

        divs.push(border);
    }

    var visibleVertically =
    (
        position.top <= vScrollValue &&
        position.top + height >= vScrollValue + totalBubbleHeight
    ) ||
    (
        position.top >= vScrollValue &&
        position.top + totalBubbleHeight <= vScrollValue + screenHeight
    );

    var visibleHorizontally =
    (
        position.left <= hScrollValue &&
        position.left + width >= hScrollValue
    ) ||
    (
        position.left >= hScrollValue &&
        position.left <= hScrollValue + screenWidth
    );

    if (!visibleVertically && autoScroll)
    {
        var extraScroll = 0;

        if (totalBubbleHeight > height)
        {
            extraScroll = (totalBubbleHeight - height) / 2 + 1;

            if (position.top < vScrollValue)
            {
                extraScroll = -extraScroll;
            }
        }


        // Setting $("body").scrollTop() is no guarantee that the page scrolled
        // all the way to that value, so we ask the page for its new scroll
        // position. -mdb 2012-03-01
        $("body").scrollTop(position.top + extraScroll);
        vScrollValue = $("body").scrollTop();
    }

    if (!visibleHorizontally && autoScroll)
    {
        var extraScroll = 0;

        if (BUBBLE_MAX_WIDTH > width)
        {
            extraScroll = (BUBBLE_MAX_WIDTH - width) / 2 + 1;

            if (position.left < hScrollValue)
            {
                extraScroll = -extraScroll;
            }
        }

        // See comment above
        $("body").scrollLeft(position.left + extraScroll);
        hScrollValue = $("body").scrollLeft();
    }

    if (hover)
    {
        if (position.top <= vScrollValue)
        {
            visibleTop = vScrollValue;
        }

        if (position.top + height >= vScrollValue + screenHeight)
        {
            visibleBottom = vScrollValue + screenHeight;
        }

        if (position.left <= hScrollValue)
        {
            visibleLeft = hScrollValue;
        }

        if (position.left + width >= hScrollValue + screenWidth)
        {
            visibleRight = hScrollValue + screenWidth;
        }

        height = visibleBottom - visibleTop;
        width  = visibleRight  - visibleLeft;
        var y = (height / 2) + visibleTop - (BUBBLE_HEIGHT / 2);

        var bubble1 = createSelectionBubble(
            y - (BUBBLE_HEIGHT / 2) - (BUBBLE_MARGIN / 2),
            BUBBLE_MAX_WIDTH,
            BUBBLE_HEIGHT,
            elementNameForIdentifier(identifier),
            $("#" + identifier),
            hover
        );

        divs.push(bubble1);
        var description = fontDescriptionForIdentifier(identifier);

        if (description.length > 40)
        {
            description = description.substr(0, 20) + "..." +
                description.substr(description.length - 16, 16);
        }

        var bubble2 = createSelectionBubble(
            y + (BUBBLE_HEIGHT / 2) + (BUBBLE_MARGIN / 2),
            BUBBLE_MAX_WIDTH,
            BUBBLE_HEIGHT,
            description,
            $("#" + identifier),
            hover
        );

        divs.push(bubble2);

        var newWidth = Math.max(
            bubble1.outerWidth(),
            bubble2.outerWidth()
        ) + BUBBLE_PADDING;

        bubble1.css("width", newWidth);
        bubble2.css("width", newWidth);

        var x = (width / 2) + visibleLeft - (newWidth / 2);
        bubble1.offset({left: x});
        bubble2.offset({left: x});
    }
    return divs;
}

function createSelectionBorder(x, y, width, height, parent)
{
    var border = document.createElement("div");
    $(border).css("border-color", pasteboardTextColor.toRgbaString());
    $(border).css("width", parseInt(width) + 4);
    $(border).css("height", parseInt(height) + 4);
    $(border).attr("id", "extensis-selection-border" + ++borderCount);

    addClass($(border), "extensis-selection-border");
    parent.append(border);
    addClass($(border), "extensis-selection-border");

    $(border).offset({ left: parseInt(x) - 4, top: parseInt(y) - 4});

    return $(border)
}

function createSelectionBubble(y, maxWidth, height, string, parent, hover)
{
    var bubble  = document.createElement("div");

    $(bubble).css("line-height", height + "px");
    $(bubble).css("max-width", maxWidth);
    $(bubble).css("height", height);
    $(bubble).attr("id", "extensis-selection-bubble" + ++bubbleCount);

    addClass($(bubble), "extensis-selection-bubble");

    if (navigator.appName != "Microsoft Internet Explorer")
    {
        $(bubble).corner("14px");
    }

    var p = document.createElement("p");
    $(p).css("width", "100%");
    $(p).css("white-space", "nowrap");
    $(p).attr("id", "extensis-selection-bubble-text" + bubbleCount);
    addClass($(p), "extensis-selection-bubble-text");
    p.appendChild(document.createTextNode(string));
    bubble.appendChild(p);

    parent.append(bubble);
    addClass($(p), "extensis-selection-bubble-text");
    addClass($(bubble), "extensis-selection-bubble");

    $(bubble).offset({top: y});

    $(bubble).fadeOut(0);
    $(bubble).fadeIn(HOVER_FADE_TIME);

    return $(bubble);
}

function selectIdentifier(identifier)
{
    selectedIdentifier = identifier;

    if (identifier != null)
    {
        hideSelectionRects();
        $("#" + identifier).find("> .extensis-selection-border").show();

        elementClicked(identifier);
    }
}

function deselectAll()
{
    hideSelectionRects();
    selectedElement = null;
    elementDeselected();
}

function hideSelectionRects()
{
    $(".extensis-selection-border").hide()
}

function makeIdentifierSelectable(identifier)
{
    $("#" + identifier).mouseenter(function()
    {
        if (showInfo)
        {
            createSelectionUIElements($(this), $(this).attr("id"), false, true);
        }
    });

    $("#" + identifier).mouseleave(function()
    {
        $(".extensis-selection-bubble").each(
        function () {
            if (this != null)
            {
                $(this).fadeOut(HOVER_FADE_TIME, function(index)
                {
                    $(this).remove();
                });
            }
        });
    });

    $("#" + identifier).click(function(event)
    {
        selectIdentifier(identifier);
        event.stopPropagation();
    });

    addClass($("#" + identifier), "extensis-selectable-element");
}


// This function returns a list of selectable identifiers in the order that
// they appear in the HTML. This is useful for e.g. the printing cover page.
function getSelectableIdentifiers()
{
    ids = new Array();
    $(".extensis-selectable-element").each(function ()
    {
        ids.push($(this).attr("id"))
    });

    return ids;
}

$(document).ready(function()
{
    $("html").click(function(event)
    {
        deselectAll();
    });

    $(window).resize(function(event)
    {
        rebuildUIElements();
    });
});

function addPrintPage(identifier)
{
    var leftMargin = parseInt($("html").css("margin-left")) +
                     parseInt($("html").css("padding-left")) +
                     parseInt($("body").css("margin-left")) +
                     parseInt($("body").css("padding-left"));

    var rightMargin = parseInt($("html").css("margin-right")) +
                      parseInt($("html").css("padding-right")) +
                      parseInt($("body").css("margin-right")) +
                      parseInt($("body").css("padding-right"));

    var topMargin = parseInt($("html").css("margin-top")) +
                    parseInt($("html").css("padding-top")) +
                    parseInt($("body").css("margin-top")) +
                    parseInt($("body").css("padding-top"));

    var header = document.createElement("div");
    header.setAttribute("class", "extensis-print-page");
    header.setAttribute("id", "extensis-print-page");
    header.setAttribute("width", "100%");
    $(header).css("margin-left", 0 - leftMargin);
    $(header).css("margin-right", 0 - rightMargin);
    $(header).css("margin-top", 0 - topMargin);
    $("body").prepend($(header));
    addClass($(header), "extensis-print-page");
}

function setPrintLabels(fontLabel,
                        typeLabel,
                        foundryLabel,
                        classLabel,
                        familyLabel,
                        versionLabel)
{
    printFontLabel    = fontLabel;
    printTypeLabel    = typeLabel;
    printFoundryLabel = foundryLabel;
    printClassLabel   = classLabel;
    printFamilyLabel  = familyLabel;
    printVersionLabel = versionLabel;
}

function clearPrintPage()
{
    $(".extensis-print-page").empty();
    printRowNum = 1;
}

function setPrintHeader(template, category, bg, fg, date, app)
{
    var rawHTML = '\
    <table class="extensis-print-header">\
      <tr>\
            <td valign="top" width="45%">\
                <p class="extensis-print-template-name"></p>\
                <p class="extensis-print-category-name"></p>\
                <p class="extensis-print-bg-color"></p>\
                <p class="extensis-print-fg-color"></p>\
            </td>\
            <td valign="top" width="35%">\
                <p class="extensis-print-quick-comp">QuickComp</p>\
                <p class="extensis-print-date"></p>\
            </td>\
            <td valign="top" width="20%" align="right">\
                <p class="extensis-print-app-title"></p>\
                <p class="extensis-print-url">www.extensis.com</p>\
            </td>\
        </tr>\
    </table>';

    header = jQuery.parseHTML(rawHTML);

    $(header).find(".extensis-print-template-name").append(
        document.createTextNode(template)
    );

    $(header).find(".extensis-print-category-name").append(
        document.createTextNode(category)
    );

    $(header).find(".extensis-print-bg-color").append(
        document.createTextNode(bg)
    );

    $(header).find(".extensis-print-fg-color").append(
        document.createTextNode(fg)
    );

    $(header).find(".extensis-print-date").append(
        document.createTextNode(date)
    );

    $(header).find(".extensis-print-app-title").append(
        document.createTextNode(app)
    );

    $(".extensis-print-page").append(header);

    var hr = document.createElement("hr");
    $(".extensis-print-page").append(hr);

    var sections = document.createElement(printSectionsType());
    sections.setAttribute("id", "extensis-print-sections");
    addClass($(sections), "extensis-print-sections");
    $(".extensis-print-page").append(sections);
    addClass($(sections), "extensis-print-sections");
}

function printSectionsType()
{
    var sections = null;

    if (isMac)
    {
        return "div"
    }
    else
    {
        return "table"
    }
}

function addPrintSection(elementIdentifier,
                         elementName,
                         fontName,
                         type,
                         foundry,
                         classification,
                         family,
                         version,
                         fakeFamily,
                         size)
{
    if (fontName.length > 30)
    {
        fontName = fontName.substr(0, 14) + "..." +
            fontName.substr(fontName.length - 15, 14);
    }

    if (family.length > 30)
    {
        family = family.substr(0, 14) + "..." +
            family.substr(family.length - 14, 14);
    }

    rawHTML = printSectionElementHTML()

    row = jQuery.parseHTML(rawHTML);

    // Despite reports to the contrary, Safari does not (as of 2013-05-20)
    // support page-break-inside: avoid. IE8 and later support it, but does so
    // inconsistently. Instead, we limit to 5 print sections per page, which is
    // the maximum number than can be guaranteed to fit on a page.
    //
    // Since we insert a page break after the table, we don't want to insert a
    // page break on the last row, or we'll end up with an empty page between
    // the print info and the template.
    //
    // https://bugs.webkit.org/show_bug.cgi?id=35217

    if (printRowNum % 6 == 0)
    {
        if (isMac)
        {
            rawHTML = '\
                <div class="extensis-print-spacer-row extensis-page-break-div"><p></p></div>';
            spacerRow = jQuery.parseHTML(rawHTML);
            $(".extensis-print-sections").append(spacerRow);
        }
        else
        {
            rawHTML = '\ <tr class="extensis-page-break-tr"><td><p></p></td></tr>';
            var pageBreakRow = jQuery.parseHTML(rawHTML);
            $(".extensis-print-sections").append(pageBreakRow);

            rawHTML = '\
                <tr class="extensis-print-spacer-row"><td><p></p></td></tr>';
            spacerRow = jQuery.parseHTML(rawHTML);
            $(".extensis-print-sections").append(spacerRow);
        }
    }

    $(row).find(".extensis-print-element").append(
        document.createTextNode(elementName)
    );

    $(".extensis-print-sections").append(row);

    rawHTML = printSectionInfoHTML();

    row = jQuery.parseHTML(rawHTML);

    preview = $(row).find(".extensis-print-preview");
    $(preview).css("font-size", size);
    $(preview).attr("id", "extensis-print-preview" + ++printPreviewCount);

    applyFontToPrintPreview(elementIdentifier, $(preview), fakeFamily);

    $(row).find(".extensis-print-font-label").append(
        document.createTextNode(printFontLabel)
    );

    $(row).find(".extensis-print-font").append(
        document.createTextNode(fontName)
    );

    $(row).find(".extensis-print-type-label").append(
        document.createTextNode(printTypeLabel)
    );

    $(row).find(".extensis-print-type").append(
        document.createTextNode(type)
    );

    $(row).find(".extensis-print-foundry-label").append(
        document.createTextNode(printFoundryLabel)
    );

    $(row).find(".extensis-print-foundry").append(
        document.createTextNode(foundry)
    );

    $(row).find(".extensis-print-class-label").append(
        document.createTextNode(printClassLabel)
    );

    $(row).find(".extensis-print-class").append(
        document.createTextNode(classification)
    );

    $(row).find(".extensis-print-family-label").append(
        document.createTextNode(printFamilyLabel)
    );

    $(row).find(".extensis-print-family").append(
        document.createTextNode(family)
    );

    $(row).find(".extensis-print-version-label").append(
        document.createTextNode(printVersionLabel)
    );

    $(row).find(".extensis-print-version").append(
        document.createTextNode(version)
    );

    $(".extensis-print-sections").append(row);

    applyFontToPrintPreview(elementIdentifier, $(preview), fakeFamily);

    ++printRowNum;
}

function printSectionElementHTML()
{
    if (isMac)
    {
        return '<div class="extensis-print-element"><p></p></div>';
    }
    else
    {
        return '\
        <tr height="24">\
            <td colspan="4" valign="bottom"><p class="extensis-print-element"></p></td>\
        </tr>';
    }
}

function printSectionInfoHTML()
{
    if (isMac)
    {
        return '\
        <div width="100%">\
        <table class="extensis-print-row">\
            <tr>\
                <td valign="top">\
                    <h1 class="extensis-print-preview">Lorem Ipsum Dolor Sit Amet</h1>\
                </td>\
                <td align ="left" valign="top" width="200">\
                    <table class="extensis-print-info">\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-font-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-font extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-type-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-type extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-foundry-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-foundry extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-class-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-class extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-family-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-family extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-version-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-version extensis-print-info-value-mac"></p>\
                        </td>\
                    </tr>\
                </table>\
                </td>\
            </tr>\
        </table>\
    </div>'
    }
    else
    {
        return '\
        <tr>\
            <td valign="top"><h1 class="extensis-print-preview">Lorem Ipsum Dolor Sit Amet</h1></td>\
            <td align="left" valign="top">\
                <table class="extensis-print-info">\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-font-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-font"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-type-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-type"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-foundry-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-foundry"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-class-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-class"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-family-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-family"></p>\
                        </td>\
                    </tr>\
                    <tr>\
                        <td align="right" width="80" valign="top">\
                            <p class="extensis-print-version-label"></p>\
                        </td>\
                        <td width="8">&nbsp;</td>\
                        <td align="left">\
                            <p class="extensis-print-version"></p>\
                        </td>\
                    </tr>\
                </table>\
            </td>\
        </tr>'
    }
}

function setShowInfo(show)
{
    showInfo = show;
}

function rebuildUIElements()
{
    $(".extensis-selection-border").remove();

    $(".extensis-selectable-element").each(function()
    {
        createSelectionUIElements($(this), $(this).attr("id"), false, false);
    });

    selectIdentifier(selectedIdentifier);
}

function windowLostFocus()
{
    $(".extensis-selection-bubble").remove();

    $("div").mouseleave();
    $("span").mouseleave();
    $("p").mouseleave();
}
